@@ -253,7 +253,7 @@ class BoxValidator:
         # file corruption), replace them  with printable character
         if bc.containsControlCharacters(boxType):
             boxType=bc.replaceControlCharacters(boxType)
-        
+
         # Decode to string with Latin encoding
         # Elementtree will deal with any non-ASCII characters by replacing
         # them with numeric entity references
@@ -667,7 +667,7 @@ class BoxValidator:
         primaryPlatform = self.boxContents[40:44]
         self.addCharacteristic("primaryPlatform", primaryPlatform)
 
-        # Profile flags (bytes 44-47; only first byte read here as remaing bytes
+        # Profile flags (bytes 44-47; only first byte read here as remaining bytes
         # don't contain any meaningful information)
         profileFlags = bc.bytesToUnsignedChar(self.boxContents[44:45])
 
@@ -689,7 +689,7 @@ class BoxValidator:
         deviceModel = self.boxContents[52:56]
         self.addCharacteristic("deviceModel", deviceModel)
 
-        # Device attributes (bytes 56-63; only first byte read here as remaing bytes
+        # Device attributes (bytes 56-63; only first byte read here as remaining bytes
         # don't contain any meaningful information)
         deviceAttributes = bc.bytesToUnsignedChar(self.boxContents[56:57])
 
@@ -1235,6 +1235,9 @@ class BoxValidator:
             lqcd = self.characteristics.findElementText('qcd/lqcd')
             qStyle = self.characteristics.findElementText('qcd/qStyle')
             levels = self.characteristics.findElementText('cod/levels')
+        else:
+            lqcd = -9998
+            qStyle = -9999
 
         # Expected lqcd as a function of qStyle and levels
         if qStyle == 0:
@@ -1257,6 +1260,10 @@ class BoxValidator:
         numberOfTilesExpected = self.characteristics.findElementText(
             'siz/numberOfTiles')
 
+        # If we did not get the number of tiles, assume it is zero
+        if not numberOfTilesExpected:
+            numberOfTilesExpected = 0
+
         # Impose upper limit on numberOfTilesExpected to avoid misbehaviour in case of corrupted files
         # Value of 65535 equals upper value imposed by Kakadu (can't find this
         # anywhere in the standard though)
@@ -1283,7 +1290,7 @@ class BoxValidator:
             marker = self.boxContents[offset:offset + 2]
 
             # TEST
-            #print("Offset: " + str(offset))
+            # print("Offset: " + str(offset))
             # TEST
 
             if marker == b'\xff\x90':
@@ -1776,21 +1783,22 @@ class BoxValidator:
         comment = self.boxContents[4:lcom]
 
         # Decode to string with Latin encoding, determine if valid ISO 8859-15
-        
+
         try:
             comment = comment.decode("iso-8859-15", "strict")
         except:
             # Empty string in case of decode error
             comment = ""
-                
+
         # Ideally decode above should raise exception if comment is not valid
         # ISO 8859-15, but this doesn't work. So instead we do this indirectly
-        # by looking for control characters (tab, newline and carriage return are OK)
+        # by looking for control characters (tab, newline and carriage return
+        # are OK)
         if bc.removeControlCharacters(comment) == comment:
             commentIsValid = True
         else:
             commentIsValid = False
-        
+
         self.testFor("commentIsValid", commentIsValid)
 
         # Only add comment to characteristics if text (may contain binary data
@@ -2198,8 +2206,8 @@ class BoxValidator:
         loc = self.boxContents[4:len(self.boxContents)]
 
         # Last byte of loc must be null terminator
-        self.testFor("locHasNullTerminator", loc[-1] == b'\x00')
-        
+        self.testFor("locHasNullTerminator", loc.endswith(b'\x00'))
+
         # Remove null character as this cannot be represented as XML
         loc = bc.removeNullTerminator(loc)
 
@@ -2209,14 +2217,14 @@ class BoxValidator:
         if bc.containsControlCharacters(loc):
             loc=bc.replaceControlCharacters(loc)
         """
-        
-        # Try decode to UTF-8 
+
+        # Try decode to UTF-8
         try:
-            tmp=loc.decode("utf-8","strict")
+            tmp = loc.decode("utf-8", "strict")
             self.testFor("locIsUTF8", True)
         except UnicodeDecodeError:
             self.testFor("locIsUTF8", False)
-        
+
         self.addCharacteristic("loc", loc)
 
     def validate_JP2(self):
