@@ -17,6 +17,7 @@
 #   Jaishree Davey, The National Archives, UK.
 #   Laura Damian, The National Archives, UK.
 #   Carl Wilson, Open Planets Foundation, UK.
+#   Stefan Weil, UB Mannheim, DE.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -32,6 +33,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 import sys
+import mmap
 import os
 import time
 import imp
@@ -39,8 +41,8 @@ import glob
 import struct
 import argparse
 import config
-import platform
 import codecs
+import re
 import etpatch as ET
 import fnmatch
 import xml.etree.ElementTree as ETree
@@ -49,12 +51,13 @@ from xml.dom import minidom
 from byteconv import bytesToText
 from shared import printWarning
 scriptPath, scriptName = os.path.split(sys.argv[0])
+from six import u
 
 # scriptName is empty when called from Java/Jython, so this needs a fix
 if len(scriptName) == 0:
     scriptName = 'jpylyzer'
 
-__version__ = "1.14.2"
+__version__ = "1.17.0"
 
 # Create parser
 parser = argparse.ArgumentParser(
@@ -88,7 +91,6 @@ def readFileBytes(file):
 
     return(fileData)
 
-
 def generatePropertiesRemapTable():
 
     # Generates nested dictionary which is used to map 'raw' property values
@@ -259,21 +261,34 @@ def generatePropertiesRemapTable():
 
     return(enumerationsMap)
 
+def fileToMemoryMap(file):
+    # Read contents of file to memory map object
+
+    # Open file
+    f = open(file, "rb")
+
+    # Call to mmap is different on Linux and Windows, so we need to know
+    # the current platform
+    platform = config.PLATFORM
+
+    try:
+        if platform == "win32":
+            # Parameters for Windows may need further fine-tuning ...
+            fileData = mmap.mmap(f.fileno(), 0, access=mmap.ACCESS_READ)
+        else:
+            # This works for Linux (and Cygwin on Windows). Not too sure
+            # about other platforms like Mac OS though
+            fileData = mmap.mmap(f.fileno(), 0, mmap.MAP_SHARED, mmap.PROT_READ)
+    except ValueError:
+        # mmap fails on empty files.
+        fileData = ""
+    
+    f.close()    
+    return(fileData)  
 
 def checkOneFile(file):
     # Process one file and return analysis result as element object
 
-    fileData = readFileBytes(file)
-    isValidJP2, tests, characteristics = BoxValidator(
-        "JP2", fileData).validate()  # validateJP2(fileData)
-
-    # Generate property values remap table
-    remapTable = generatePropertiesRemapTable()
-
-    # Create printable version of tests and characteristics tree
-    tests.makeHumanReadable()
-    characteristics.makeHumanReadable(remapTable)
-
     # Create output elementtree object
 
     if config.inputRecursiveFlag or config.inputWrapperFlag:
@@ -282,51 +297,91 @@ def checkOneFile(file):
         root = ET.Element('jpylyzer')
     else:
         root = ET.Element(
-        'jpylyzer', {'xmlns' : 'http://openpreservation.org/ns/jpylyzer/',
-        'xmlns:xsi' : 'http://www.w3.org/2001/XMLSchema-instance' ,
-        'xsi:schemaLocation' : 'http://openpreservation.org/ns/jpylyzer/ http://jpylyzer.openpreservation.org/jpylyzer-v-1-0.xsd'})
+            'jpylyzer', {'xmlns': 'http://openpreservation.org/ns/jpylyzer/',
+                         'xmlns:xsi': 'http://www.w3.org/2001/XMLSchema-instance',
+                         'xsi:schemaLocation': 'http://openpreservation.org/ns/jpylyzer/ http://jpylyzer.openpreservation.org/jpylyzer-v-1-1.xsd'})
 
-    # Create elements for storing tool and file meta info
+    # Create elements for storing tool, file and status meta info
     toolInfo = ET.Element('toolInfo')
     fileInfo = ET.Element('fileInfo')
+    statusInfo = ET.Element('statusInfo')
+
+    # File name and path 
+    fileName = os.path.basename(file)
+    filePath = os.path.abspath(file)
+
+    # If file name / path contain any surrogate pairs, remove them to
+    # avoid problems when writing to XML
+    fileNameCleaned = stripSurrogatePairs(fileName)
+    filePathCleaned = stripSurrogatePairs(filePath)
 
-    # File name and path may contain non-ASCII characters, decoding to Latin should
-    # (hopefully) prevent any Unicode decode errors. Elementtree will then deal with any non-ASCII
-    # characters by replacing them with numeric entity references
-    try:
-        # This works in Python 2.7, but raises error in 3.x (no decode
-        # attribute for str type!)
-        fileName = os.path.basename(file).decode("iso-8859-15", "strict")
-        filePath = os.path.abspath(file).decode("iso-8859-15", "strict")
-    except AttributeError:
-        # This works in Python 3.x, but goes wrong withh non-ASCII chars in 2.7
-        fileName = os.path.basename(file)
-        filePath = os.path.abspath(file)
 
     # Produce some general tool and file meta info
     toolInfo.appendChildTagWithText("toolName", scriptName)
     toolInfo.appendChildTagWithText("toolVersion", __version__)
-    fileInfo.appendChildTagWithText("fileName", fileName)
-    fileInfo.appendChildTagWithText("filePath", filePath)
+    fileInfo.appendChildTagWithText("fileName", fileNameCleaned)
+    fileInfo.appendChildTagWithText("filePath", filePathCleaned)
     fileInfo.appendChildTagWithText(
         "fileSizeInBytes", str(os.path.getsize(file)))
+    try:
+        lastModifiedDate = time.ctime(os.path.getmtime(file))
+    except ValueError:
+        # Dates earlier than 1 Jan 1970 can raise ValueError on Windows
+        # Workaround: replace by lowest possible value (typically 1 Jan 1970)
+        lastModifiedDate = time.ctime(0)
     fileInfo.appendChildTagWithText(
-        "fileLastModified", time.ctime(os.path.getmtime(file)))
+        "fileLastModified", lastModifiedDate)
 
-    # Append to root
+    # Initialise success flag
+    success = True
+    
+    try:
+        # Contents of file to memory map object
+        fileData = fileToMemoryMap(file)
+        isValidJP2, tests, characteristics = BoxValidator("JP2", fileData).validate()
+        
+        if fileData != "":
+            fileData.close()
+
+        # Generate property values remap table
+        remapTable = generatePropertiesRemapTable()
+
+        # Create printable version of tests and characteristics tree
+        tests.makeHumanReadable()
+        characteristics.makeHumanReadable(remapTable)
+    except Exception as ex:    
+        isValidJP2 = False
+        success = False
+        exceptionType = type(ex)
+
+        if exceptionType == MemoryError:
+            failureMessage = "memory error (file size too large)"
+        elif exceptionType == IOError:
+            failureMessage = "I/O error (cannot open file)"
+        elif exceptionType == RuntimeError:
+            failureMessage = "runtime error (please report to developers)"
+        else:
+            failureMessage = "unknown error (please report to developers)"
+
+        printWarning(failureMessage)
+        tests = ET.Element("tests")
+        characteristics = ET.Element('properties')
+ 
+    # Add status info
+    statusInfo.appendChildTagWithText("success", str(success))
+    if success == False:
+        statusInfo.appendChildTagWithText("failureMessage",failureMessage)
+  
+    # Append all results to root
     root.append(toolInfo)
     root.append(fileInfo)
-
-    # Add validation outcome
+    root.append(statusInfo)
     root.appendChildTagWithText("isValidJP2", str(isValidJP2))
-
-    # Append test results and characteristics to root
     root.append(tests)
     root.append(characteristics)
 
     return(root)
 
-
 def checkNullArgs(args):
     # This method checks if the input arguments list and exits program if
     # invalid or no input argument is supplied.
@@ -353,6 +408,45 @@ def printHelpAndExit():
     parser.print_help()
     sys.exit()
 
+def stripSurrogatePairs(ustring):
+
+    # Removes surrogate pairs from a Unicode string
+
+    # This works for Python 3.x, but not for 2.x!
+    # Source: http://stackoverflow.com/q/19649463/1209004
+
+    if config.PYTHON_VERSION.startswith(config.PYTHON_3):
+        try:
+            ustring.encode('utf-8')
+        except UnicodeEncodeError:
+            # Strip away surrogate pairs
+            tmp = ustring.encode('utf-8', 'surrogateescape')
+            ustring = tmp.decode('utf-8', 'ignore')
+
+    # In Python 2.x we need to use regex
+    # Source: http://stackoverflow.com/a/18674109/1209004
+
+    if config.PYTHON_VERSION.startswith(config.PYTHON_2):
+        # Generate regex for surrogate pair detection
+
+        lone = re.compile(
+            u(r"""(?x)            # verbose expression (allows comments)
+            (                    # begin group
+            [\ud800-\udbff]      #   match leading surrogate
+            (?![\udc00-\udfff])  #   but only if not followed by trailing surrogate
+            )                    # end group
+            |                    #  OR
+            (                    # begin group
+            (?<![\ud800-\udbff]) #   if not preceded by leading surrogate
+            [\udc00-\udfff]      #   match trailing surrogate
+            )                   # end group
+            """))
+   
+        # Remove surrogates (i.e. replace by empty string) 
+        tmp = lone.sub(r'',ustring).encode('utf-8')
+        ustring = tmp.decode('utf-8')
+
+    return(ustring)
 
 def getFilesFromDir(dirpath):
     for fp in os.listdir(dirpath):
@@ -380,7 +474,6 @@ def getFilesWithPatternFromTree(rootDir, pattern):
             searchpattern = os.path.join(thisDirectory, pattern)
             getFiles(searchpattern)
 
-
 def getFilesFromTree(rootDir):
     # Recurse into directory tree and return list of all files
     # NOTE: directory names are disabled here!!
@@ -401,6 +494,10 @@ def findFiles(recurse, paths):
 
     # process the list of input paths
     for root in paths:
+    
+        if config.PYTHON_VERSION.startswith(config.PYTHON_2):
+            # Convert root to UTF-8 (only needed for Python 2.x)
+            root = unicode(root, 'utf-8')
 
         # WILDCARD IN PATH OR FILENAME
         # In Linux wilcard expansion done by bash so, add file to list
@@ -430,10 +527,10 @@ def findFiles(recurse, paths):
             """
 
             # If the input path returned files list, add files to List
-            
+
             if len(filesList) == 1 and os.path.isfile(filesList[0]):
                 existingFiles.append(filesList[0])
-            
+
             if len(filesList) > 1:
                 for f in filesList:
                     if os.path.isfile(f):
@@ -485,6 +582,7 @@ def findFiles(recurse, paths):
             elif os.path.isdir(root):
                 getFilesFromTree(root)
 
+
 def writeElement(elt, codec):
     # Writes element as XML to stdout using defined codec
 
@@ -529,7 +627,7 @@ def checkFiles(recurse, wrap, paths):
 
     # If there are no valid input files then exit program
     checkNoInput(existingFiles)
-
+    
     # Set encoding of the terminal to UTF-8
     if config.PYTHON_VERSION.startswith(config.PYTHON_2):
         out = codecs.getwriter(config.UTF8_ENCODING)(sys.stdout)
@@ -541,7 +639,7 @@ def checkFiles(recurse, wrap, paths):
         xmlHead = "<?xml version='1.0' encoding='UTF-8'?>\n"
         xmlHead += "<results xmlns=\"http://openpreservation.org/ns/jpylyzer/\"\n"
         xmlHead += "xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\"\n"
-        xmlHead += "xsi:schemaLocation=\"http://openpreservation.org/ns/jpylyzer/ http://jpylyzer.openpreservation.org/jpylyzer-v-1-0.xsd\">\n"
+        xmlHead += "xsi:schemaLocation=\"http://openpreservation.org/ns/jpylyzer/ http://jpylyzer.openpreservation.org/jpylyzer-v-1-1.xsd\">\n"
     else:
         xmlHead = "<?xml version='1.0' encoding='UTF-8'?>\n"
     out.write(xmlHead)
@@ -551,7 +649,7 @@ def checkFiles(recurse, wrap, paths):
 
         # Analyse file
         xmlElement = checkOneFile(path)
-
+       
         # Write output to stdout
         writeElement(xmlElement, out)
 
